package org.unitedfront2.domain;

import java.util.Date;

import org.unitedfront2.dao.SimpleFindable;
import org.unitedfront2.dao.SimpleUserDao;

/**
 * [Ue[uW[łB
 *
 * @author kurokkie
 *
 */
public class SimpleUserTable extends SimpleTableModule<SimpleUser> {

    /** ƔfłsԊui~bj */
    private long activeInterSpace = SimpleUser.DEFAULT_ACTIVE_INTERSPACE;

    /** [Uf[^ANZXIuWFNg */
    private SimpleUserDao simpleUserDao;

    @Override
    protected SimpleFindable<SimpleUser> getSimpleDao() {
        return simpleUserDao;
    }

    /**
     * [UCXg[܂B
     *
     * @param user [U
     * @require ${user.id} not null.
     * @require ${user.code} not null.
     * @require ${user.name} not null.
     * @require ${user.id} not exists.
     * @ensure ${this.find(user.id)} equals ${user}
     * @throws UserCodeUsedByOtherException R[hd
     * @throws UserNameUsedByOtherException Od
     */
    public void install(SimpleUser user)
        throws UserCodeUsedByOtherException, UserNameUsedByOtherException {
        if (findByCode(user.getCode()) != null) {
            if (logger.isWarnEnabled()) {
                logger.warn("User code '" + user.getCode()
                    + "' already exists.");
            }
            throw new UserCodeUsedByOtherException(user.getCode());
        }

        if (findByName(user.getName()) != null) {
            if (logger.isWarnEnabled()) {
                logger.warn("User name '" + user.getName()
                    + "' already exists.");
            }
            throw new UserNameUsedByOtherException(user.getName());
        }
        simpleUserDao.register(user);
    }

    /**
     * w肵R[h[U݂邩ǂ肵܂B
     *
     * @param code R[h
     * @return ݂ꍇ <code>true</code> A݂Ȃꍇ <code>false</code>
     */
    public boolean existsByCode(String code) {
        return simpleUserDao.findByCode(code) != null;
    }

    /**
     * w肵O[U݂邩ǂ肵܂B
     *
     * @param name O
     * @return ݂ꍇ <code>true</code> A݂Ȃꍇ <code>false</code>
     */
    public boolean existsByName(String name) {
        return simpleUserDao.findByName(name) != null;
    }

    /**
     * w肵R[h[U擾܂B
     *
     * @param code R[h
     * @return [UAȂꍇ <code>null</code>
     */
    public SimpleUser findByCode(String code) {
        return simpleUserDao.findByCode(code);
    }

    /**
     * w肵O[U擾܂B
     *
     * @param name O
     * @return [U
     */
    public SimpleUser findByName(String name) {
        return simpleUserDao.findByName(name);
    }

    /**
     * w肵R[h[U擾܂B
     *
     * @param code R[h
     * @return [U
     * @throws IllegalArgumentException [UȂ
     */
    public SimpleUser getByCode(String code) throws IllegalArgumentException {
        SimpleUser user = findByCode(code);
        if (user == null) {
            String message = "The code '" + code + "' not found.";
            logger.error(message);
            throw new IllegalArgumentException(message);
        }
        return user;
    }

    /**
     * w肵O[U擾܂B
     *
     * @param name O
     * @return [U
     * @throws IllegalArgumentException [UȂ
     */
    public SimpleUser getByName(String name) throws IllegalArgumentException {
        SimpleUser user = findByName(name);
        if (user == null) {
            String message = "The name '" + name + "' not found.";
            logger.error(message);
            throw new IllegalArgumentException(message);
        }
        return user;
    }

    /**
     * ̃[U擾܂B̃[UƂ́Aw肵Ȍ̍ŏIANZX[
     * UłB
     *
     * @return [U
     */
    public int countActiveUser() {
        Date currentDate = simpleUserDao.getCurrentDate();
        return simpleUserDao.countByLastAccessDateAfter(new Date(
                currentDate.getTime() - activeInterSpace));
    }

    public void setActiveInterSpace(long activeInterSpace) {
        this.activeInterSpace = activeInterSpace;
    }

    public void setSimpleUserDao(SimpleUserDao simpleUserDao) {
        this.simpleUserDao = simpleUserDao;
    }
}
